@@ -9,8 +9,8 @@
 namespace dbarts {
   struct Data {
     const double* y;
-    const double* X;
-    const double* X_test;
+    const double* x;
+    const double* x_test;
     
     const double* weights;
     const double* offset;
@@ -25,14 +25,14 @@ namespace dbarts {
     const std::uint32_t* maxNumCuts; // length = numPredictors if control.useQuantiles is true
     
     Data() :
-      y(NULL), X(NULL), X_test(NULL), weights(NULL), offset(NULL), testOffset(NULL),
+      y(NULL), x(NULL), x_test(NULL), weights(NULL), offset(NULL), testOffset(NULL),
       numObservations(0), numPredictors(0), numTestObservations(0),
       sigmaEstimate(1.0), variableTypes(NULL), maxNumCuts(NULL)
     { }
     
     Data(const double* y,
-         const double* X,
-         const double* X_test,
+         const double* x,
+         const double* x_test,
          const double* weights,
          const double* offset,
          const double* testOffset,
@@ -42,7 +42,7 @@ namespace dbarts {
          double sigmaEstimate,
          const VariableType* variableTypes,
          const std::uint32_t* maxNumCuts) :
-      y(y), X(X), X_test(X_test), weights(weights), offset(offset), testOffset(testOffset),
+      y(y), x(x), x_test(x_test), weights(weights), offset(offset), testOffset(testOffset),
       numObservations(numObservations), numPredictors(numPredictors), numTestObservations(numTestObservations),
       sigmaEstimate(sigmaEstimate), variableTypes(variableTypes), maxNumCuts(maxNumCuts)
     {
